@@ -3,11 +3,14 @@ delete_1_to_x <- function(ds, p, cols_mis, cols_ctrl, x,
                           cutoff_fun = median,
                           prop = 0.5, use_lpSolve = TRUE,
                           ordered_as_unordered = FALSE,
-                          stochastic = FALSE,
-                          add_realized_x = FALSE, ...) {
+                          n_mis_stochastic = FALSE,
+                          x_stochastic = FALSE,
+                          add_realized_x = FALSE,
+                          warn_p = getOption("missMethods.warn.too.high.p"),
+                          ...) {
 
-  # general checking is done in calling functions delete_MAR_1_to_x and
-  # delete_MNAR_1_to_x, only special cases are checked here
+  # General checking of arguments is done in delete_values().
+  # Only special cases are checked here.
   # check if cols_ctrl are numeric or ordered factor
   check_cols_ctrl_1_to_x(ds, cols_ctrl)
 
@@ -21,7 +24,18 @@ delete_1_to_x <- function(ds, p, cols_mis, cols_ctrl, x,
     stop("x must be greater than 0 and finite")
   }
 
-  p <- adjust_p(p, cols_mis)
+  # Check x_stochastic
+  stopifnot(is.logical(x_stochastic), length(x_stochastic) == 1L)
+  if (!x_stochastic && n_mis_stochastic) {
+    warning(
+      "x_stochastic is set to TRUE because x_stochastic = FALSE",
+      " is only meaningful for n_mis_stochastic = FALSE.",
+      " If you want x_stochastic = FALSE, set n_mis_stochastic = FALSE, which is",
+      " TRUE right now."
+    )
+    x_stochastic <- TRUE
+  }
+
 
   # create missing values -----------------------
   n <- nrow(ds)
@@ -33,13 +47,12 @@ delete_1_to_x <- function(ds, p, cols_mis, cols_ctrl, x,
     )
     if (is.null(groups$g2)) {
       warning("column ", cols_ctrl[i], " is constant, effectively MCAR")
-      ds[, cols_mis[i]] <- delete_MCAR_vec(
-        ds[, cols_mis[i], drop = TRUE],
-        p[i], stochastic
-      )
-      true_odds[i] <- 0
-    } else {
-
+      na_indices <- get_NA_indices(n_mis_stochastic, n = nrow(ds), p = p[i])
+    } else if (x_stochastic) {
+      prob <- rep(1, nrow(ds))
+      prob[groups$g2] <- x
+      na_indices <- get_NA_indices(n_mis_stochastic, nrow(ds), p = p[i], prob = prob)
+    } else if (!n_mis_stochastic) {
       # calculate p_mis for group 1 and group 2
       nr_g1 <- length(groups$g1)
       nr_g2 <- length(groups$g2)
@@ -48,54 +61,46 @@ delete_1_to_x <- function(ds, p, cols_mis, cols_ctrl, x,
       # check if p_mis_g1 or p_mis_g2 is out of range (>1)
       if (p_mis_g2 > 1) {
         x_max_i <- nr_g2 / (n * p[i] - nr_g1)
-        warning(
-          "p (or x) is too high; x is set to ", x_max_i,
-          " to get expected n * p missing values"
-        )
+        if (warn_p) {
+          warning(
+            "p (or x) is too high; x is set to ", x_max_i,
+            " to get expected n * p missing values"
+          )
+        }
         p_mis_g2 <- 1 # setting x = x_max_i results in p_mis_g2 = 1
         p_mis_g1 <- (p[i] * n - nr_g2) / nr_g1
       } else if (p_mis_g1 > 1) {
         x_min_i <- (n * p[i] - nr_g2) / nr_g1
-        warning(
-          "p is too high or x to low; x is set to ", x_min_i,
-          " to get expected n * p missing values"
-        )
+        if (warn_p) {
+          warning(
+            "p is too high or x to low; x is set to ", x_min_i,
+            " to get expected n * p missing values"
+          )
+        }
         p_mis_g1 <- 1
         p_mis_g2 <- (p[i] * n - nr_g1) / nr_g2
       }
 
       # delete values
-      if (stochastic) { # stochastic = TRUE ------------------
-        na_indices_g1 <- groups$g1[sample(c(TRUE, FALSE),
-          nr_g1,
-          replace = TRUE,
-          prob = c(
-            p_mis_g1,
-            1 - p_mis_g1
-          )
-        )]
-        na_indices_g2 <- groups$g2[sample(c(TRUE, FALSE),
-          nr_g2,
-          replace = TRUE,
-          prob = c(
-            p_mis_g2,
-            1 - p_mis_g2
-          )
-        )]
-      } else { # stochastic = FALSE ------------------
-        nr_mis <- round(p[i] * n)
-        nr_mis_g1 <- calc_nr_mis_g1(nr_g1, p_mis_g1, nr_g2, nr_mis, x)
-        nr_mis_g2 <- nr_mis - nr_mis_g1
+      if (n_mis_stochastic) { # n_mis_stochastic = TRUE ------------------
+        na_indices_g1 <- get_NA_indices(n_mis_stochastic, p = p_mis_g1, indices = groups$g1)
+        na_indices_g2 <- get_NA_indices(n_mis_stochastic, p = p_mis_g2, indices = groups$g2)
+      } else { # n_mis_stochastic = FALSE ------------------
+        n_mis <- round(p[i] * n)
+        n_mis_g1 <- calc_n_mis_g1(nr_g1, p_mis_g1, nr_g2, n_mis, x)
+        n_mis_g2 <- n_mis - n_mis_g1
 
         # sample NA indices
-        na_indices_g2 <- resample(groups$g2, nr_mis_g2)
-        na_indices_g1 <- resample(groups$g1, nr_mis_g1)
+        na_indices_g1 <- get_NA_indices(n_mis_stochastic, n_mis = n_mis_g1, indices = groups$g1)
+        na_indices_g2 <- get_NA_indices(n_mis_stochastic, n_mis = n_mis_g2, indices = groups$g2)
       }
       na_indices <- c(na_indices_g1, na_indices_g2)
-      ds[na_indices, cols_mis[i]] <- NA
-      true_odds[i] <- (length(na_indices_g1) / nr_g1) /
-        (length(na_indices_g2) / nr_g2)
+    } else {
+      stop("something went wrong. Please contact maintainer.")
     }
+    ds[na_indices, cols_mis[i]] <- NA
+    true_odds[i] <- sum(na_indices %in% groups$g1) * length(groups$g2) /
+      (length(groups$g1) * sum(na_indices %in% groups$g2))
   }
 
   if (add_realized_x) {
@@ -107,6 +112,25 @@ delete_1_to_x <- function(ds, p, cols_mis, cols_ctrl, x,
 }
 
 
+check_cols_ctrl_1_to_x <- function(ds, cols_ctrl) {
+  # check if cols_ctrl are numeric or ordered factor
+  cols_prob <- integer(0)
+  for (k in seq_along(cols_ctrl)) {
+    if (!(is.ordered(ds[, cols_ctrl[k], drop = TRUE]) |
+      is.numeric(ds[, cols_ctrl[k], drop = TRUE]))) {
+      cols_prob <- c(cols_prob, cols_ctrl[k])
+    }
+  }
+  if (length(cols_prob) > 0L) {
+    stop(
+      "all cols_ctrl must be numeric or ordered factors;\n",
+      "problematic column(s): ",
+      paste(cols_prob, collapse = ", ")
+    )
+  }
+  TRUE
+}
+
 
 
 #' Create MAR values using MAR1:x
@@ -140,7 +164,7 @@ delete_1_to_x <- function(ds, p, cols_mis, cols_ctrl, x,
 #' But there are some restrictions, which can lead to some deviations from the
 #' odds 1:x (see below).
 #'
-#' If \code{stochastic = FALSE} (the default),
+#' If \code{x_stochastic} and \code{n_mis_stochastic} are false (the default),
 #' then exactly \code{round(nrow(ds) * p[i])} values will be set \code{NA} in
 #' column \code{cols_mis[i]}.
 #' To achieve this, it is possible that the true odds differ from 1:x.
@@ -148,13 +172,15 @@ delete_1_to_x <- function(ds, p, cols_mis, cols_ctrl, x,
 #' chosen to minimize the absolute difference between the realized odds and 1:x.
 #' Furthermore, if \code{round(nrow(ds) * p[i])} == 0, then no missing value
 #' will be created in \code{cols_mis[i]}.
-#' If \code{stochastic = TRUE}, the number of missing values in
-#' \code{cols_mis[i]} is a random variable.
-#' This random variable is a sum of two binomial distributed variables (one for
-#' group 1 and one for group 2).
-#' If \code{p} is not too high and \code{x} is not too high or to low (see
-#' below), then the odds 1:x will be met in expectation.
-#' But in a single dataset the odds will be unequal to 1:x most of the time.
+#'
+#' If \code{x_stochastic} is true, the rows from the two groups will get
+#' sampling weights proportional to 1 (group 1) and x (group 2). If
+#' \code{n_mis_stochastic} is false, these weights are given to
+#' \code{\link{sample}} via the argument \code{prob} and exactly
+#' \code{round(nrow(ds) * p[i])} values will be set \code{NA}. If
+#' \code{n_mis_stochastic} is true, the sampling weights will be scaled and
+#' compared to uniform random numbers. The scaling is done in such a way to get
+#' expected \code{nrow(ds) * p[i]} missing values in \code{cols_mis[i]}.
 #'
 #' If \code{p} is high and \code{x} is too high or too low, it is possible that
 #' the odds 1:x and the proportion of missing values \code{p} cannot be
@@ -165,16 +191,17 @@ delete_1_to_x <- function(ds, p, cols_mis, cols_ctrl, x,
 #' If a combination of \code{p} and \code{x} that cannot be realized together
 #' is given to \code{delete_MAR_1_to_x}, then a warning will be generated and
 #' \code{x} will be adjusted in such a way that \code{p} can be realized as
-#' given to the function.
+#' given to the function. The warning can be silenced by setting the option
+#' \code{missMethods.warn.too.high.p} to false.
 #'
 #' The argument \code{add_realized_x} controls whether the x of the realized
 #' odds are added to the return value or not.
 #' If \code{add_realized_x = TRUE}, then the realized x values for all
 #' \code{cols_mis} will be added as an attribute to the returned object.
-#' For \code{stochastic = TRUE} these realized x will differ from the given
+#' For \code{x_stochastic = TRUE} these realized x will differ from the given
 #' \code{x} most of the time and will change if the function is rerun without
 #' setting a seed.
-#' For \code{stochastic = FALSE}, it is also possible that the realized odds
+#' For \code{x_stochastic = FALSE}, it is also possible that the realized odds
 #' differ (see above). However, the realized odds will be constant over multiple
 #' runs.
 #'
@@ -191,6 +218,7 @@ delete_1_to_x <- function(ds, p, cols_mis, cols_ctrl, x,
 #'   groups, if \code{cols_ctrl[i]} is an unordered factor.
 #' @param ordered_as_unordered Logical; should ordered factors be treated as
 #'   unordered factors.
+#' @param x_stochastic Logical; should the odds be stochastic or deterministic.
 #' @param ... Further arguments passed to \code{cutoff_fun}.
 #'
 #' @export
@@ -199,7 +227,7 @@ delete_1_to_x <- function(ds, p, cols_mis, cols_ctrl, x,
 #' @examples
 #' ds <- data.frame(X = 1:20, Y = 101:120)
 #' delete_MAR_1_to_x(ds, 0.2, "X", "Y", 3)
-#' # beware of small datasets and stochastic = FALSE
+#' # beware of small datasets and x_stochastic = FALSE
 #' attr(delete_MAR_1_to_x(ds, 0.4, "X", "Y", 3, add_realized_x = TRUE), "realized_x")
 #' attr(delete_MAR_1_to_x(ds, 0.4, "X", "Y", 4, add_realized_x = TRUE), "realized_x")
 #' attr(delete_MAR_1_to_x(ds, 0.4, "X", "Y", 5, add_realized_x = TRUE), "realized_x")
@@ -208,35 +236,20 @@ delete_1_to_x <- function(ds, p, cols_mis, cols_ctrl, x,
 #' # either 6 above 2 below (x = 3) or
 #' # 7 above and 1 below (x = 7)
 #' # Too high combination of p and x:
-#' delete_MAR_1_to_x(ds, 0.9, "X", "Y", 3)
-#' delete_MAR_1_to_x(ds, 0.9, "X", "Y", 3, stochastic = TRUE)
+#' tryCatch(delete_MAR_1_to_x(ds, 0.9, "X", "Y", 3), warning = function(w) w)
 delete_MAR_1_to_x <- function(ds, p, cols_mis, cols_ctrl, x,
                               cutoff_fun = median,
                               prop = 0.5,
                               use_lpSolve = TRUE,
                               ordered_as_unordered = FALSE,
-                              stochastic = FALSE,
+                              n_mis_stochastic = FALSE,
+                              x_stochastic = FALSE,
                               add_realized_x = FALSE, ...,
-                              miss_cols, ctrl_cols) {
-
-  # Deprecate miss_cols, ctrl_cols
-  check_renamed_arg(miss_cols, cols_mis)
-  check_renamed_arg(ctrl_cols, cols_ctrl)
-
-  check_delete_args_MAR(
-    ds = ds, p = p, cols_mis = cols_mis,
-    cols_ctrl = cols_ctrl, stochastic = stochastic
-  )
-
-  delete_1_to_x(ds, p, cols_mis, cols_ctrl,
-    x = x,
-    cutoff_fun = cutoff_fun,
-    prop = prop,
-    use_lpSolve = use_lpSolve,
-    ordered_as_unordered = ordered_as_unordered,
-    stochastic = stochastic,
-    add_realized_x = add_realized_x, ...
-  )
+                              miss_cols, ctrl_cols, stochastic) {
+  do.call(delete_values, c(
+    list(mechanism = "MAR", mech_type = "1_to_x"),
+    as.list(environment()), list(...)
+  ))
 }
 
 
@@ -255,25 +268,12 @@ delete_MNAR_1_to_x <- function(ds, p, cols_mis, x,
                                prop = 0.5,
                                use_lpSolve = TRUE,
                                ordered_as_unordered = FALSE,
-                               stochastic = FALSE,
+                               n_mis_stochastic = FALSE,
+                               x_stochastic = FALSE,
                                add_realized_x = FALSE, ...,
-                               miss_cols) {
-
-  # Deprecate miss_cols
-  check_renamed_arg(miss_cols, cols_mis)
-
-  check_delete_args_MNAR(
-    ds = ds, p = p, cols_mis = cols_mis,
-    stochastic = stochastic
-  )
-
-  delete_1_to_x(ds, p, cols_mis,
-    cols_ctrl = cols_mis, x = x,
-    cutoff_fun = cutoff_fun,
-    prop = prop,
-    use_lpSolve = use_lpSolve,
-    ordered_as_unordered = ordered_as_unordered,
-    stochastic = stochastic, add_realized_x = add_realized_x,
-    ...
-  )
+                               miss_cols, stochastic) {
+  do.call(delete_values, c(
+    list(mechanism = "MNAR", mech_type = "1_to_x"),
+    as.list(environment()), list(...)
+  ))
 }
