@@ -1,11 +1,3 @@
-test_that("delete_MAR_rank() calls check_delete_args_MAR()", {
-  expect_error(
-    delete_MAR_rank(df_XY_100, 0.1, 1, cols_ctrl = 3),
-    "indices in cols_ctrl must be in 1:ncol\\(ds)"
-  )
-})
-
-
 test_that("delete_MAR_rank() and delete_rank() works", {
   set.seed(12345)
 
@@ -25,6 +17,13 @@ test_that("delete_MAR_rank() and delete_rank() works", {
   # pbinom(566, 1000, 2/3) + pbinom(765, 1000, 2/3, lower.tail = FALSE)
   # 2.683384e-11 # not very likely...
 
+  # check n_mis_stochastic
+
+  set.seed(12345)
+  N <- 1000
+  n_mis <- mean(replicate(N, count_NA(delete_MAR_rank(df_XY_20, 0.125, "X", "Y", n_mis_stochastic = TRUE)["X"])))
+  expect_true(2 < n_mis, n_mis < 3)
+
 
   # check special ctrl_col cases
   # ctrl_col constant
@@ -75,12 +74,6 @@ test_that("delete_MAR_rank() (and delete_rank(), which is called by
 
 # check delete_MNAR_rank -----------------------------
 test_that("delete_MNAR_rank() works", {
-  # check that delete_MNAR_rank() calls check_delete_args_MNAR()
-  expect_error(
-    delete_MNAR_rank(df_XY_X_mis, 0.1, "X"),
-    "cols_mis must be completely observed; no NAs in ds\\[, cols_mis\\] allowed"
-  )
-
   df_mis <- delete_MNAR_rank(df_XY_100, c(0.3, 0.1), c("X", "Y"))
   expect_equal(count_NA(df_mis), c(X = 30, Y = 10))
 })
