@@ -15,13 +15,13 @@ test_that("evaluate_imputed_values()", {
       df_XY_20,
       criterion = "MAE",
       M = matrix(c(TRUE, rep(FALSE, 39)), nrow = 20),
-      which_cols = 1
+      cols_which = 1
     ),
     1
   )
 
   expect_error(
     evaluate_imputed_values(df_XY_20_imp, df_XY_100),
-    "the dimensions of imp_ds and orig_ds must be equal"
+    "the dimensions of ds_imp and ds_orig must be equal"
   )
 })
